/***************************************************************************
                          FPConversion.c  -  description
                             -------------------
    begin                : Fri Jul 20 2001
    copyright            :
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* Implements functions for converting doubles to and from strings. */

#include "FPConversion.h"

typedef unsigned long long UInt64;


/* Helper function for DoubleToString that converts a UInt64 into a string. 
 * s         - Pointer to Char * that is moved along as the number is converted.
 * n         - The number to be converted.
 * minDigits - Minimum number of digits - the number is padded with leading
 *             zeros if necessary.
 * Does NOT null-terminate the string; on completion, *s points to the end
 * of the converted string.
 */
static void UInt64ToString(Char **s, UInt64 n, Int16 minDigits)
{
	static Char stack[50];
	Char *p;

	/* Handle special case of n == 0 */
	if(n == 0)
		*(*s)++ = '0';
	else
	{
		/* Push the digits onto a stack so that they can be reversed */
		p = stack;
	 	while(n > 0)
		{
			*p++ = '0'+(n%10);
			n /= 10;
			minDigits--;
		}
		/* Pad with zeros */
		while(minDigits-- > 0)
			*p++ = '0';
		/* Pop the digits off the stack in the correct order */
		while(p > stack)
			*(*s)++ = *(--p);
	}
} 


/* Converts a double to a string.
 * buf      - The buffer to place the string in. If NULL, a statically 
 *            allocated buffer is used, which will be overwritten on the next 
 *            call to this function.
 * d        - The number to be converted.
 * decimals - The number of decimals to include.
 * Returns  - A pointer to the buffer containing the string. 
 */
Char *DoubleToString(Char *buf, double d, UInt8 decimals)
{
	/* Statically allocated buffer used when buf == NULL */
	static Char static_buf[50];
	
	/* Counter */
	UInt8 i;
	
	/* Pointer into the buffer being used */
	Char *p;
	
	/* Flag indicating whether the number is negative */
	Boolean negative = (d < 0.0);
	
	/* Absolute value of the number */
	double absd = negative ? -d : d;
	
	/* Left-hand side and right-hand side of the decimal point */
	UInt64 lhs, rhs;
	
	/* Intermediate value 10^decimals */
	UInt64 ten_to_decimals = 1;
	for(i = 0; i < decimals; i++)
		ten_to_decimals *= 10;
	
	/* Add (0.5 / 10^decimals) to the number for rounding off the final digit */
	absd += (double) 0.5 / (double) ten_to_decimals;
	
	if(decimals > 0)
	{
		/* Find lhs and rhs when decimals > 0 */
		double fraction;
		
		lhs = (UInt64) absd;
		fraction = absd-(double) lhs;
		/* Extract the right-hand-side digits by setting 
		 * rhs = (fraction * 10^decimals) */
		rhs = (UInt64) (fraction * (double) ten_to_decimals);
	}
	else
		/* Handle easy case with no decimals */
		lhs = (UInt64) absd;

	/* Now that we have the sign, lhs and rhs, build the string */
	if(buf == NULL)
		/* Use static_buf if buf == NULL */
		buf = static_buf;
	p = buf;

	if(negative)
		*p++ = '-';
	UInt64ToString(&p, lhs, 0); 
	if(decimals > 0)
	{
		*p++ = '.';
		UInt64ToString(&p, rhs, decimals);
	}
	
	/* Null-terminate the string and return it */
	*p++ = 0;
	return buf;	
}


/* Converts a string to a double. 
 * str     - The string to be converted.
 * d       - Pointer to double in which to place the result.
 * Returns - false on success - true on error.
 */
Boolean StringToDouble(Char *str, double *d)
{
	/* Whether or not the number is negative */
	Boolean negative = (*str == '-');
	
	/* Skip the minus sign if the number is negative */ 
	if(negative)
		str++;

	/* It is an error if the string is empty or just "-" */
	if(*str == 0)
		return true; /* error */
		
	/* Convert the digits to the left of the decimal sign */
	*d = 0;
	while(*str != '.' && *str != 0)
	{
		if(*str < '0' || *str > '9')
			return true; /* error */
		*d = *d * 10 + (*str - '0');
		str++;
	}
	
	/* Convert the digits to the right of the decimal sign (if any) */
	if(*str == '.')
	{
		double fraction = 0;
		
		/* This is easiest if we start from the end - so we scan to the end of
		 * the string while we make sure that all the characters are digits.
		 */ 
		Char *end = ++str;
		while(*end != 0) {
			if(*end < '0' || *end > '9')
				return true; /* error */
			end++;
		}
		/* Now end points to the end of the string (the final 0 character).
		 * Backtrack and calculate the fraction part of the number.
		 */
		while(end > str)
			fraction = (fraction+(*(--end) - '0'))/10;
		/* Add the fraction part of the number */
		*d += fraction;
	}
	
	/* Apply the correct sign */
	if(negative)
		*d = - *d;
		
	/* Everything's OK */
	return false;
}
