/***************************************************************************
                          PabiloDB.c
                             -------------------
    begin                : Fri Jul 20 2001
    copyright            : (C) 2001 by Michiel van Turnhout
    email                : info@qris.myweb.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <PalmOS.h>
#include "Pabilo.h"
#include "PabiloDB.h"

//Started in Vaals
/***********************************************************************
 *
 * FUNCTION:     SetDBBackupBit
 *
 * DESCRIPTION:  This routine sets the backup bit on the given database.
 *					     This is to aid syncs with non Palm software.
 *					     If no DB is given, open the app's default database and set
 *					     the backup bit on it.
 *
 * PARAMETERS:   dbP -	the database to set backup bit,
 *							 can be NULL to indicate app's default database
 *
 * RETURNED:     nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *
 ***********************************************************************/
void SetDBBackupBit(DmOpenRef dbP)
{
	DmOpenRef localDBP;
	LocalID dbID;
	UInt16 cardNo;
	UInt16 attributes;

	// Open database if necessary. If it doesn't exist, simply exit (don't create it).
	if (dbP == NULL)
	{
		localDBP = DmOpenDatabaseByTypeCreator (pabiloDBType, sysFileCPabilo, dmModeReadWrite);
		if (localDBP == NULL)  return;
	}
	else
	{
		localDBP = dbP;
	}

	// now set the backup bit on localDBP
	DmOpenDatabaseInfo(localDBP, &dbID, NULL, NULL, &cardNo, NULL);
	DmDatabaseInfo(cardNo, dbID, NULL, &attributes, NULL, NULL,
				   NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	attributes |= dmHdrAttrBackup;
	DmSetDatabaseInfo(cardNo, dbID, NULL, &attributes, NULL, NULL,
					  NULL, NULL, NULL, NULL, NULL, NULL, NULL);

	// close database if necessary
	if (dbP == NULL)
	{
		DmCloseDatabase(localDBP);
	}
}

/************************************************************
 *
 *  FUNCTION: PabiloLocalizeAppInfo
 *
 *  DESCRIPTION: Look for localize app info strings and copy
 *  them into the app info block.
 *
 *  PARAMETERS: application info ptr
 *
 *  RETURNS: nothing
 *
 *
 *************************************************************/
static void PabiloLocalizeAppInfo (PabiloAppInfoPtr appInfoP)
{
	Int16 		  i;
	Char **		  stringsP;
	MemHandle 	stringsH;
	MemHandle 	localizedAppInfoH;
	Char *		  localizedAppInfoP;
	PabiloAppInfoPtr	nilP = 0;


	localizedAppInfoH = DmGetResource(appInfoStringsRsc, LocalizedAppInfoStr);
	if (localizedAppInfoH)
		{
		localizedAppInfoP = MemHandleLock(localizedAppInfoH);
		stringsH = SysFormPointerArrayToStrings(localizedAppInfoP, 
			dmRecNumCategories);
		stringsP = MemHandleLock(stringsH);
		
		// Copy each category
		for (i = 0; i < dmRecNumCategories; i++)
			{
			if (stringsP[i][0] != '\0')
				DmStrCopy(appInfoP, (UInt32) nilP->categoryLabels[i], stringsP[i]);
			}
		
		MemPtrFree(stringsP);
		MemPtrUnlock(localizedAppInfoP);
		}
}


/************************************************************
 *
 *  FUNCTION: PabiloAppInfoInit
 *
 *  DESCRIPTION: Create an app info chunk if missing.  Set
 *		the strings to a default.
 *
 *  PARAMETERS: database pointer
 *
 *  RETURNS: 0 if successful, errorcode if not
 *
 *
 *************************************************************/
Err	PabiloAppInfoInit(DmOpenRef dbP)
{
	UInt16 				    cardNo;
	MemHandle			    h;
	LocalID 				  dbID;
	LocalID 				  appInfoID;
	PabiloAppInfoPtr 	nilP = 0;
	PabiloAppInfoPtr	appInfoP;
	
	if (DmOpenDatabaseInfo(dbP, &dbID, NULL, NULL, &cardNo, NULL))
		return dmErrInvalidParam;
	if (DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, 
			NULL, NULL, &appInfoID, NULL, NULL, NULL))
		return dmErrInvalidParam;
	
	if (appInfoID == 0)
		{
		h = DmNewHandle (dbP, sizeof (PabiloAppInfoType));
		if (! h) return dmErrMemError;
		
		appInfoID = MemHandleToLocalID (h);
		DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &appInfoID, NULL, NULL, NULL);
		}
	
	appInfoP = MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof(PabiloAppInfoType), 0);

	// Initialize the categories.
	CategoryInitialize ((AppInfoPtr) appInfoP, LocalizedAppInfoStr);

	// Initialize the sort order.
	DmSet (appInfoP, (UInt32)&nilP->sortOrder, sizeof(appInfoP->sortOrder), 
		soTimeStamp);

	MemPtrUnlock(appInfoP);

	// The conduit ignores dmHdrAttrAppInfoDirty
	return 0;
}


/************************************************************
 *
 *  FUNCTION:    PabiloGetAppInfo
 *
 *  DESCRIPTION: Get the app info chunk 
 *
 *  PARAMETERS:  database pointer
 *
 *  RETURNS:     MemHandle to the to do application info block 
 *               (PabiloAppInfoType)
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			
 *
 *************************************************************/
static MemHandle PabiloGetAppInfo (DmOpenRef dbP)
{
	Err error;
	UInt16 cardNo;
	LocalID dbID;
	LocalID appInfoID;
	
	error = DmOpenDatabaseInfo (dbP, &dbID, NULL, NULL, &cardNo, NULL);
	ErrFatalDisplayIf (error,  "Get getting app info block");
	
	error = DmDatabaseInfo (cardNo, dbID, NULL, NULL, NULL, NULL, NULL, 
			NULL, NULL, &appInfoID, NULL, NULL, NULL);
	ErrFatalDisplayIf (error,  "Get getting app info block");

	return ((MemHandle) MemLocalIDToGlobal (appInfoID, cardNo));
}



/************************************************************
 *
 *  FUNCTION:     PabiloNewRecord
 *
 *  DESCRIPTION:  Create a new record
 *
 *  PARAMETERS:   database pointer
 *				        database record
 *
 *  RETURNS: zero if successful, errorcode if not
 *
 *  REVISION HISTORY:
 *			Name	Date		      Description
 *			----	----		      -----------
 *			MvT	  26-04-2002    Initial Revision
 *
 *************************************************************/
Err PabiloNewRecord (DmOpenRef dbP, PabiloItemType *recordData)
{
	Err 					  result;
	UInt32			    size = 0;
  PabiloDBRecordPtr  newRecordPtr;
  UInt16          recordIndex;
  MemHandle       newRecordHandle;

	recordIndex = DmNumRecords(dbP);
	// Compute the size of the new record.
  size = sizeof(PabiloDBRecordType);
	//  Allocate a chunk in the database for the new record.
  newRecordHandle = (MemHandle) DmNewRecord( dbP,&recordIndex,(UInt32) size);
	if (newRecordHandle == NULL)
		return dmErrMemError;
  newRecordPtr = MemHandleLock(newRecordHandle);

  DmWrite(newRecordPtr,0,recordData,size);

  MemPtrUnlock(newRecordPtr);
  DmReleaseRecord(dbP, recordIndex, true);
	
	return result;
}

/***********************************************************************
 *
 * FUNCTION:     PabiloGetDatabase
 *
 * DESCRIPTION:  Get the application's database.  Open the database if it
 * exists, create it if neccessary.
 *
 * PARAMETERS:   *dbPP - pointer to a database ref (DmOpenRef) to be set
 *					  mode - how to open the database (dmModeReadWrite)
 *
 * RETURNED:     Err - zero if no error, else the error
 *
 * REVISION HISTORY:
 *			Name		Date		Description
 *			----		----		-----------
 *
 ***********************************************************************/
Err PabiloGetDatabase (DmOpenRef *dbPP,Char *dbName, UInt16 mode) {
	Err error = 0;
	DmOpenRef dbP;
	UInt16 cardNo = 0;
	LocalID dbID;

	*dbPP = NULL;

  // Find the application's data file.  If it doesn't exist create it.
  dbID = DmFindDatabase(cardNo, dbName);
  if(dbID == NULL) {
		// database does not exist create it
		//error = DmGetLastErr();
		error = DmCreateDatabase (cardNo, dbName, sysFileCPabilo, pabiloDBType, false);
		if (error)
			return error;
		dbID = DmFindDatabase(cardNo, dbName);
	}
	
	if(dbID) {
		dbP = DmOpenDatabase(cardNo,dbID,mode);
		if (!dbP)
			return (1);
	}
		
		// Set the backup bit.  This is to aid syncs with non Palm software.
	SetDBBackupBit(dbP);


	*dbPP = dbP;
	return error;
}

Boolean DBExists(Char *dbName) {
	Err error = 0;
	UInt16 cardNo;
	LocalID dbID;
	Boolean retval = false;

	// Find the application's data file.  If it doesn't exist return false.
  dbID = DmFindDatabase(cardNo, dbName);
  if(dbID == NULL) 
		retval = false;
	else
		retval = true;
	return retval;
}

Err PabiloDeleteDB(Char *dbName) {
	Err error = 0;
	UInt16 cardNo = 0;
	LocalID dbID = NULL;
	Boolean retval = false;

	// Find the application's data file.  If it doesn't exist return false.
  dbID = DmFindDatabase(cardNo, dbName);
  if(dbID)
		error = DmDeleteDatabase(cardNo,dbID);
	return error;
}

